import re

import requests

from sota_extractor.errors import HttpClientError
from sota_extractor.scrapers.utils import date_from_timestamp, sround
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB


SQUAD_URL = "https://rajpurkar.github.io/SQuAD-explorer/"
SQUAD_MASTER_URL = (
    "https://raw.githubusercontent.com/rajpurkar/SQuAD-explorer/master"
)
SQUAD_1_1_JSON_URL = f"{SQUAD_MASTER_URL}/out-v1.1.json"
SQUAD_2_0_JSON_URL = f"{SQUAD_MASTER_URL}/out-v2.0.json"

TASK_NAME = "Question Answering"
DATASET_1_NAME = "SQuAD1.1"
DATASET_1_DESCRIPTION = (
    "Previous version of the SQuAD2.0 dataset with 50,000 question-answer "
    "pairs."
)
DATASET_2_NAME = "SQuAD2.0"
DATASET_2_DESCRIPTION = (
    "Stanford Question Answering Dataset (SQuAD) is a reading comprehension "
    "dataset, consisting of questions posed by crowdworkers on a set of "
    "Wikipedia articles, where the answer to every question is a segment of "
    "text, or span, from the corresponding reading passage, or the question "
    "might be unanswerable."
)


def get_sota_rows(data):
    rows = data["leaderboard"]

    sota_rows = []
    for row in rows:
        # HACK: This hack with the timezone is needed because they don't use
        #       timezones. They just run the gulp html generation script in
        #       localtime which is (I guess) Stanford i.e. US/Pacific zone so
        #       we need to parse the timestamp like weare in that zone to get
        #       the same dates they get.
        date = date_from_timestamp(
            row.get("submission", {}).get("created", None), tz="US/Pacific"
        )

        description = row.get("submission", {}).get("description", "").strip()
        # This peace of a the code is taken from gulpfile.js translated to py
        regex_match = re.match(r"(.*?) ?\((.*?)\) ?\((.*?)\)(.*)", description)
        if regex_match is not None:
            groups = regex_match.groups()
            model_name = f"{groups[0].strip()} ({groups[1].strip()})"
            # _institution = groups[2].strip()
            if groups[3].find("http") != -1:
                link = groups[3].strip()
            else:
                link = ""
        else:
            model_name = description[: description.rfind("(")].strip()
            # _first_part = description[description.rfind("(") + 1 :]
            # _institution = _first_part[: _first_part.rfind(")")]
            if description.find("http") != -1:
                link = description[description.rfind("http") :].strip()
            else:
                link = ""

        em = row.get("scores", {}).get("exact_match", None)
        f1 = row.get("scores", {}).get("f1", None)

        # Skip rows with no values
        if em is None or f1 is None:
            continue

        sota_rows.append(
            SotaRow(
                model_name=model_name,
                paper_title=link,
                paper_url=link,
                paper_date=date,
                metrics={"EM": sround(em, 3), "F1": sround(f1, 3)},
            )
        )
    return sota_rows


def squad() -> TaskDB:
    """Extract SQUAD SOTA tables."""
    try:
        squad_1 = requests.get(SQUAD_1_1_JSON_URL).json()
        squad_2 = requests.get(SQUAD_2_0_JSON_URL).json()
    except Exception as e:
        raise HttpClientError(message=str(e))

    dataset1 = Dataset(
        name=DATASET_1_NAME,
        is_subdataset=False,
        description=DATASET_1_DESCRIPTION,
    )
    dataset2 = Dataset(
        name=DATASET_2_NAME,
        is_subdataset=False,
        description=DATASET_2_DESCRIPTION,
    )
    task = Task(name=TASK_NAME)
    task.datasets = [dataset2, dataset1]
    task.source_link = Link(title="SQuAD Leaderboard", url=SQUAD_URL)

    # scrape the evaluation values on the two datasets
    dataset1.sota.metrics = ["EM", "F1"]
    dataset2.sota.metrics = ["EM", "F1"]

    dataset1.sota.rows = get_sota_rows(squad_1)
    dataset2.sota.rows = get_sota_rows(squad_2)

    tdb = TaskDB()
    tdb.add_task(task)
    return tdb



from datetime import datetime

import requests

from sota_extractor.scrapers.utils import sround
from sota_extractor.errors import HttpClientError
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB


URL_2018 = "http://ymcui.com/cmrc2018/"
URL_2019 = "http://ymcui.com/cmrc2019/"

CMRC_2018_JSON_URL = (
    "https://raw.githubusercontent.com/ymcui/cmrc2018/gh-pages/docs/"
    "raw_score.json"
)
CMRC_2019_JSON_URL = (
    f"https://raw.githubusercontent.com/ymcui/cmrc2019/gh-pages/docs/"
    f"raw_score.json"
)

TASK_NAME = "Chinese Reading Comprehension"
DATASET_2018_NAME = "CMRC 2018 (Chinese Machine Reading Comprehension 2018)"
DATASET_2019_NAME = "CMRC 2019 (Chinese Machine Reading Comprehension 2019)"


def get_sota_rows(data):
    sota_rows = []

    if data["version"] == "cmrc2018":
        for row in data["data"]:
            try:
                date = datetime.strptime(row.get("date", ""), "%b %d, %Y")
            except ValueError:
                date = None

            model_name = row.get("system_name", "").strip()
            model_type = row.get("model_type", "").strip()
            if model_type != "":
                model_name = f"{model_name} ({model_type})"

            test_em = row.get("test_em", None)
            test_f1 = row.get("test_f1", None)
            challenge_em = row.get("challenge_em", None)
            challenge_f1 = row.get("challenge_f1", None)

            # Skip rows with no values
            if (
                test_em is None
                or test_f1 is None
                or challenge_em is None
                or challenge_f1 is None
            ):
                continue

            sota_rows.append(
                SotaRow(
                    model_name=model_name,
                    paper_title=row["paper"],
                    paper_url="" if row["url"] == "-" else row["url"],
                    paper_date=date,
                    metrics={
                        "TEST-EM": sround(test_em, 3),
                        "TEST-F1": sround(test_f1, 3),
                        "CHALLENGE-EM": sround(challenge_em, 3),
                        "CHALLENGE-F1": sround(challenge_f1, 3),
                    },
                )
            )

    elif data["version"] == "cmrc2019":
        for row in data["data"]:
            try:
                date = datetime.strptime(row.get("date", ""), "%Y/%m/%d")
            except ValueError:
                date = None

            model_name = row.get("system_name", "").strip()
            model_type = row.get("model_type", "").strip()
            if model_type != "":
                model_name = f"{model_name} ({model_type})"

            sota_rows.append(
                SotaRow(
                    model_name=model_name,
                    paper_title=row["url"],
                    paper_url=row["url"],
                    paper_date=date,
                    metrics={
                        "QAC": str(row.get("test_qac", 0)),
                        "PAC": str(row.get("test_pac", 0)),
                    },
                )
            )

    return sota_rows


def cmrc() -> TaskDB:
    """Extract CMRC SOTA tables."""
    try:
        cmrc_2018 = requests.get(CMRC_2018_JSON_URL).json()
        cmrc_2019 = requests.get(CMRC_2019_JSON_URL).json()
    except Exception as e:
        raise HttpClientError(message=str(e))

    dataset_2018 = Dataset(
        name=DATASET_2018_NAME,
        is_subdataset=False,
        links=[Link("CMRC 2018 Leadergboard", url=URL_2018)],
    )
    dataset_2019 = Dataset(
        name=DATASET_2019_NAME,
        is_subdataset=False,
        links=[Link("CMRC 2019 Leadergboard", url=URL_2019)],
    )

    task = Task(name=TASK_NAME)

    task.datasets = [dataset_2018, dataset_2019]

    # scrape the evaluation values on the two datasets
    dataset_2018.sota.metrics = [
        "TEST-EM",
        "TEST-F1",
        "CHALLENGE-EM",
        "CHALLENGE-F1",
    ]
    dataset_2019.sota.metrics = ["QAC", "PAC"]

    dataset_2018.sota.rows = get_sota_rows(cmrc_2018)
    dataset_2019.sota.rows = get_sota_rows(cmrc_2019)

    tdb = TaskDB()
    tdb.add_task(task)
    return tdb



import re
import json
import _jsonnet
from datetime import datetime

import requests

from sota_extractor.taskdb.v01 import (
    Link,
    Task,
    Dataset,
    Sota,
    SotaRow,
    TaskDB,
)


XTREME_URL = "https://sites.research.google/xtreme/app.js"


REGEX = re.compile(
    r"""
    ^                                  # Start of the line
    \s*                                # Optional space
    passageLbList\.push\(              # Push to a list
        (?P<json>\{.*?\})              # JS object that's pushed
    \)\;                               # End of the push
    \s*                                # Optional space
    $                                  # End of the line
""",
    re.VERBOSE,
)


def xtreme() -> TaskDB:
    """Extract Xtreme SOTA tables."""
    data = requests.get(XTREME_URL).text.splitlines()

    sota_rows = []
    for line in data:
        match = REGEX.match(line)
        if match is not None:
            try:
                data = json.loads(
                    _jsonnet.evaluate_snippet(
                        "snippet", match.groupdict().get("json", "{}")
                    )
                )

                # Skip human performance
                if data["modelName"].strip() == "":
                    continue

                sota_rows.append(
                    SotaRow(
                        model_name=data["modelName"],
                        paper_title=data["participant"],
                        paper_url=data["referenceLink"],
                        paper_date=datetime.strptime(
                            data["date"], "%b %d, %Y"
                        ),
                        metrics={
                            "Avg": data["avgAllScoreString"],
                            "Sentence-pair Classification": data[
                                "avgSpcScoreString"
                            ],
                            "Structured Prediction": data["avgSpScoreString"],
                            "Question Answering": data["avgQaScoreString"],
                            "Sentence Retrieval": data["avgSrScoreString"],
                        },
                    )
                )
            except Exception:
                continue

    task = Task(
        name="Zero-Shot Cross-Lingual Transfer",
        datasets=[
            Dataset(
                name="XTREME",
                is_subdataset=False,
                sota=Sota(
                    metrics=[
                        "Avg",
                        "Sentence-pair Classification",
                        "Structured Prediction",
                        "Question Answering",
                        "Sentence Retrieval",
                    ],
                    rows=sota_rows,
                ),
            )
        ],
        source_link=Link(
            title=(
                "(X) Cross-Lingual Transfer Evaluation of Multilingual "
                "Encoders"
            ),
            url="https://sites.research.google/xtreme",
        ),
    )
    tdb = TaskDB()
    tdb.add_task(task)
    return tdb



import logging
from datetime import datetime

from sota_extractor.scrapers.utils import get_soup
from sota_extractor.taskdb.v01 import (
    Link,
    Task,
    Dataset,
    SotaRow,
    TaskDB,
)


logger = logging.getLogger(__name__)


DATA = [
    {
        "url": "https://ogb.stanford.edu/docs/leader_nodeprop/",
        "datasets": [
            "ogbn-products",
            "ogbn-proteins",
            "ogbn-arxiv",
            "ogbn-papers100M",
            "ogbn-mag",
        ],
        "task": "Node Property Prediction",
    },
    {
        "url": "https://ogb.stanford.edu/docs/leader_linkprop/",
        "datasets": [
            "ogbl-ppa",
            "ogbl-collab",
            "ogbl-ddi",
            "ogbl-citation2",
            "ogbl-wikikg2",
            "ogbl-biokg",
        ],
        "task": "Link Property Prediction",
    },
    {
        "url": "https://ogb.stanford.edu/docs/leader_graphprop/",
        "datasets": ["ogbg-molhiv", "ogbg-molpcba", "ogbg-ppa", "ogbg-code2"],
        "task": "Graph Property Prediction",
    },
]


def get_sota_rows(url: str, dataset: Dataset):
    try:
        soup = get_soup(url)

        h3 = soup.find(id=f"leaderboard-for-{dataset.name}".lower())
        table = h3.find_next_sibling("table")

        headers = table.find_all("th")
        metric_1 = headers[2].text
        metric_2 = headers[3].text
        metric_3 = "Number of params"

        dataset.sota.metrics = [metric_1, metric_2, metric_3]
        for tr in table.select("tbody > tr"):
            tds = tr.find_all("td")

            try:
                paper_url = tds[5].find("a", text="Paper").attrs["href"]
            except (AttributeError, KeyError):
                paper_url = ""

            try:
                paper_date = datetime.strptime(tds[8].text, "%b %d, %Y")
            except ValueError:
                paper_date = None

            try:
                link = Link(url=tds[5].find("a", text="Code").attrs["href"])
                code_links = [link]
            except (AttributeError, KeyError):
                code_links = []

            dataset.sota.rows.append(
                SotaRow(
                    model_name=tds[1].text,
                    paper_url=paper_url,
                    paper_date=paper_date,
                    code_links=code_links,
                    metrics={
                        metric_1: tds[2].text,
                        metric_2: tds[3].text,
                        metric_3: tds[6].text.replace(",", ""),
                    },
                )
            )
    except Exception as e:
        logger.exception(
            "Failed to get dataset: %s. Error: %s", dataset.name, e
        )


def ogb() -> TaskDB:
    """Extract OGB SOTA tables."""
    tdb = TaskDB()

    for item in DATA:
        url = item["url"]

        task = Task(
            name=item["task"],
            source_link=Link(title=f"{item['task']} LeaderBoard", url=url),
        )
        tdb.add_task(task)
        for dataset_name in item["datasets"]:
            dataset = Dataset(name=dataset_name)
            task.datasets.append(dataset)
            get_sota_rows(url, dataset)
    return tdb



import sys
import argparse

from sota_extractor import serialization
from sota_extractor.scrapers.nlp_progress.printer import print_task
from sota_extractor.scrapers.nlp_progress.markdown import parse_file


def main(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("filename")
    parser.add_argument(
        "-j",
        "--json",
        action="store_true",
        default=False,
        help="Print the taskdb json output.",
    )
    ns = parser.parse_args(args)

    tdb = parse_file(ns.filename)

    if ns.json:
        print(serialization.dumps(tdb))
    else:
        for task in tdb.tasks.values():
            print_task(task)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



from sota_extractor.errors import DataError
from sota_extractor.scrapers.utils import get_soup
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB

CITYSCAPES_URL = (
    "https://www.cityscapes-dataset.com/benchmarks/#pixel-level-results"
)


TASK_NAME = "Semantic Segmentation"
DATASET_NAME = "Cityscapes test"
DATASET_DESCRIPTION = (
    "The first Cityscapes task involves predicting a per-pixel semantic "
    "labeling of the image without considering higher-level object instance "
    "or boundary information."
)


def get_sota_rows(table):
    rows = table.findAll("tr")

    sota_rows = []
    for row in rows:
        if row.find("th"):
            # Skip the header row
            continue
        cells = row.findAll("td")
        if len(cells) == 24:
            model_name = row.find("td", attrs={"class": "column-1"}).text
            paper_title = row.find("td", attrs={"class": "column-21"}).text
            if row.find("td", attrs={"class": "column-21"}).a:
                paper_url = row.find("td", attrs={"class": "column-21"}).a[
                    "href"
                ]
            else:
                paper_url = None
            class_iou = row.find("td", attrs={"class": "column-14"}).text
            category_iou = row.find("td", attrs={"class": "column-16"}).text

            if paper_title and class_iou and category_iou:
                sota_row = SotaRow(
                    model_name=model_name,
                    paper_title=paper_title,
                    paper_url=paper_url,
                    paper_date=None,
                    metrics={
                        "Mean IoU (class)": class_iou,
                        "Mean IoU (category)": category_iou,
                    },
                )
                sota_rows.append(sota_row)

    return sota_rows


def cityscapes() -> TaskDB:
    """Extract Cityscapes SOTA tables."""
    soup = get_soup(CITYSCAPES_URL)

    sota_tabels = soup.findAll("table", attrs={"class": "tablepress"})

    if len(sota_tabels) == 3:

        cityscapes = sota_tabels[0]  # pixel-level semantic segmentation task

        dataset = Dataset(
            name=DATASET_NAME,
            is_subdataset=False,
            description=DATASET_DESCRIPTION,
        )

        task = Task(name="Semantic Segmentation")
        task.datasets = [dataset]
        task.source_link = Link(
            title="CityScapes Leaderboard", url=CITYSCAPES_URL
        )

        # scrape the evaluation values on the two datasets
        dataset.sota.metrics = ["Mean IoU (class)", "Mean IoU (class)"]

        dataset.sota.rows = get_sota_rows(cityscapes)

        tdb = TaskDB()
        tdb.add_task(task)
        return tdb
    else:
        raise DataError("Got an unexpected number of SOTA tables.")



import requests
from datetime import datetime
from typing import Optional, Union

import pytz
from bs4 import BeautifulSoup


def get_soup(url):
    """Get a BeautifulSoup object back from the a URL.

    Args:
        url: URL to scrape.
    """

    r = requests.get(url)

    if r.status_code == 404:
        return None

    data = r.text
    return BeautifulSoup(data, "lxml")


def date_from_timestamp(
    timestamp: Optional[Union[int, str]], tz: str = "UTC"
) -> Optional[datetime]:
    """Get date from int or string timestamp.

    Args:
        timestamp: Timestamp to parse.
        tz: Optional timezone to use.
    """
    if timestamp is None:
        return None

    if isinstance(timestamp, str):
        try:
            timestamp = int(timestamp)
        except ValueError:
            return None

    try:
        return datetime.fromtimestamp(timestamp, pytz.timezone(tz)).replace(
            tzinfo=pytz.utc
        )
    except (TypeError, ValueError):
        return None


def sround(
    x: Union[int, float, str], ndigits: int = 3, default: float = 0.0
) -> str:
    """Round the number and return as a string.

    Args:
        x (int|float|str): Number to format.
        ndigits (int): Number of digits to enforce.
        default (float): Default value if rounding fails.

    Returns:
        str: Rounded string representation.
    """
    try:
        x = round(float(x), ndigits=ndigits)
    except ValueError:
        x = default

    return "{{:.{}f}}".format(ndigits).format(x)



import textwrap
from typing import List

from sota_extractor.taskdb.v01 import Task, Dataset


def wrap_lines(value, indent=4) -> List[str]:
    prefix = " " * indent
    return [
        textwrap.indent(line, prefix=prefix) for line in textwrap.wrap(value)
    ]


def sub_items(title, items, lines_func) -> List[str]:
    lines = []
    if len(items) == 0:
        lines.append(f"{title}: -")
    else:
        lines.append(f"{title}:")
        for item in items:
            for i, line in enumerate(lines_func(item)):
                if i == 0:
                    lines.append(f"      - {line}")
                else:
                    lines.append(f"        {line}")
    return lines


def dataset_lines(dataset: Dataset) -> List[str]:
    lines = [f"Name: {dataset.name}"]
    if dataset.description is None or dataset.description.strip() == "":
        lines.append(f"Description: -")
    else:
        lines.append(f"Description:")
        lines.extend(wrap_lines(dataset.description))
    lines.append(f"Rows: {len(dataset.sota.rows)}")

    lines.extend(sub_items("Subdatasets", dataset.subdatasets, dataset_lines))
    return lines


def task_lines(task: Task) -> List[str]:
    lines = [f"Name: {task.name}"]
    if task.description is None or task.description.strip() == "":
        lines.append(f"Description: -")
    else:
        lines.append(f"Description:")
        lines.extend(wrap_lines(task.description))
    if task.parent is None:
        lines.append(f"Parent: -")
    else:
        lines.append(f"Parent: {task.parent.name}")
    if task.source_link is None:
        lines.append(f"Source Link: -")
    else:
        lines.append(f"Source Link: {task.source_link}")

    lines.extend(
        sub_items(
            title="Datasets", items=task.datasets, lines_func=dataset_lines
        )
    )

    lines.extend(
        sub_items(title="Subtasks", items=task.subtasks, lines_func=task_lines)
    )
    return lines


def print_task(task: Task):
    """Print a task.

    Used for debugging purposes.
    """
    lines = task_lines(task)
    print("\n".join(lines))
